using System;
using System.Drawing;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;

namespace UCCellDemo
{
	/// <summary>
	/// Sample to show how to implement a UserControl based cell
	/// </summary>
	/// 

	public class MyUserControlCellModel : GridGenericControlCellModel
	{
		public MyUserControlCellModel(GridModel parentGridModel) : base(parentGridModel){}

		public override GridCellRendererBase CreateRenderer(GridControlBase grid)
		{
			return (new MyUserControlCellRenderer(grid,this));
		}

	}

	public class MyUserControlCellRenderer : GridGenericControlCellRenderer
	{

		private DemoUserControl activeControl;
		private DemoUserControl staticControl;

		public MyUserControlCellRenderer(GridControlBase grid, GridCellModelBase model):base(grid,model)
		{
			activeControl=new DemoUserControl();
      staticControl=new DemoUserControl();
			grid.DefaultRowHeight=activeControl.Height;
      grid.DefaultColWidth=activeControl.Width;
			MakeUnBuffered(staticControl);
		}

		private void MakeUnBuffered(Control c)
		{
			System.Reflection.MethodInfo mInfo = c.GetType().GetMethod("SetStyle", System.Reflection.BindingFlags.Instance |
				System.Reflection.BindingFlags.InvokeMethod |
				System.Reflection.BindingFlags.NonPublic);

			if (!(mInfo == null))
			{
				mInfo.Invoke(c, new object[]{ControlStyles.DoubleBuffer, false});
			}

			if (c.Controls.Count>0)
			{
				foreach (Control c1 in c.Controls)
				{
					c1.CausesValidation = false;
					MakeUnBuffered(c1); // recursive call
				}
			}
		}

		protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
		{
			clientRectangle.Inflate(-1,-1);

			if (ShouldDrawFocused(rowIndex,colIndex))
				style.Control=activeControl;
			else 
				style.Control=staticControl;

			base.OnDraw(g,clientRectangle,rowIndex,colIndex,style);
		}
 		

	}
}
